Attribute VB_Name = "PhVintCurInp"
Option Explicit

' ==========================================================================
' Functions:
' ==========================================================================
   
   ' ========================================================================
   ' PhidgetCurrentInput_create(
   '     PhidgetCurrentInputHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Creates an instance of a Phidget channel.  Once created the channel may
   ' need to be configured to match the physical device and channel index you
   ' want to attach to, and then be opened by calling Phidget_open().  The
   ' Phidget channel handle must be released with PhidgetCurrentInput_delete().
   '
   Public Declare Function PhCurInpCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_create" ( _
      ByRef handle&)

   ' ========================================================================
   ' PhidgetCurrentInput_delete(
   '     PhidgetCurrentInputHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Releases a Phidget channel handle created by PhidgetCurrentInput_create().
   ' If the channel is open when PhidgetCurrentInput_delete() is called, it
   ' will be closed.
   '
   Public Declare Function PhCurInpDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_delete" ( _
      ByRef handle&)

   ' =======================================================================
   ' PhidgetCurrentInput_getCurrent(
   '     PhidgetEncoderHandle ch,
   '     double *current
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     current: The current value
   ' Return: The Phidget return code
   ' Unit: amperes (A)
   '
   ' The most recent current value that the channel has reported.
   ' This value will always be between MinCurrent and MaxCurrent .
   '
   Public Declare Function PhCurInpGetCurrent& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getCurrent" ( _
      ByVal handle&, _
      ByRef current#)

   ' =======================================================================
   ' PhidgetCurrentInput_getMinCurrent()
   '     PhidgetReturnCode PhidgetCurrentInput_getMinCurrent(
   '     PhidgetCurrentInputHandle  ch,
   '     double *minCurrent
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     MinCurrent: The current value
   ' Return: The Phidget return code
   ' Constant: -30.0 A
   '
   ' The minimum value the CurrentChange event will report.
   '
   Public Declare Function PhCurInpGetMinCurrent& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMinCurrent" ( _
      ByVal handle&, _
      ByRef minCurrent#)


   ' =======================================================================
   ' PhidgetCurrentInput_getMaxCurrent()
   '     PhidgetReturnCode PhidgetCurrentInput_getMaxCurrent(
   '     PhidgetCurrentInputHandle  ch,
   '     double *maxCurrent
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     maxCurrent: The current value
   ' Return: The Phidget return code
   ' Constant: -30.0 A
   '
   ' The maximum value the CurrentChange event will report.
   '
   Public Declare Function PhCurInpGetMaxCurrent& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMaxCurrent" ( _
      ByVal handle&, _
      ByRef maxCurrent#)

   ' =======================================================================
   ' PhidgetCurrentInput_getCurrentChangeTrigger()
   '     PhidgetReturnCode PhidgetCurrentInput_getCurrentChangeTrigger(
   '     PhidgetCurrentInputHandle  ch,
   '     double *currentChangeTrigger
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     CurrentChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0.0 A
   '
   ' The channel will not issue a CurrentChange event until the current value
   ' has changed by the amount specified by the CurrentChangeTrigger.
   '
   ' Setting the CurrentChangeTrigger to 0 will result in the channel firing
   ' events every DataInterval.  This is useful for applications that
   ' implement their own data filtering.
   '
    Public Declare Function PhCurInpGetCurrentChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getCurrentChangeTrigger" ( _
      ByVal handle&, _
      ByRef currentChangeTrigger#)

   ' =======================================================================
   ' PhidgetCurrentInput_setCurrentChangeTrigger()
   '     PhidgetReturnCode PhidgetCurrentInput_setCurrentChangeTrigger(
   '     PhidgetCurrentInputHandle  ch,
   '     double  currentChangeTrigger
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     currentChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0.0 A
   '
   ' The channel will not issue a CurrentChange event until the current
   ' value has changed by the amount specified by the CurrentChangeTrigger.
   '
   ' Setting the CurrentChangeTrigger to 0 will result in the channel firing
   ' events every DataInterval.  This is useful for applications that
   ' implement their own data filtering
   '
   Public Declare Function PhCurInpSetCurrentChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_setCurrentChangeTrigger" ( _
      ByVal handle&, _
      ByVal currentChangeTrigger#)

   ' =======================================================================
   ' PhidgetCurrentInput_getMinCurrentChangeTrigger()
   '     PhidgetReturnCode PhidgetCurrentInput_getMinCurrentChangeTrigger(
   '     PhidgetCurrentInputHandle  ch,
   '     double *minCurrentChangeTrigger
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     MinCurrentChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 0.0 A
   '
   ' The minimum value that CurrentChangeTrigger can be set to.
   '
   Public Declare Function PhCurInpGetMinCurrentChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMinCurrentChangeTrigger" ( _
      ByVal handle&, _
      ByRef minCurrentChangeTrigger#)

   ' =======================================================================
   ' PhidgetCurrentInput_getMaxCurrentChangeTrigger()
   '     PhidgetReturnCode
   '     PhidgetCurrentInput_getMaxCurrentChangeTrigger(
   '     PhidgetCurrentInputHandle  ch,
   '     double *maxCurrentChangeTrigger
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     MaxCurrentChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 60.0 A
   '
   ' The maximum value that CurrentChangeTrigger can be set to.
   '
    Public Declare Function PhCurInpGetMaxCurrentChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMaxCurrentChangeTrigger" ( _
      ByVal handle&, _
      ByRef maxCurrentChangeTrigger#)

   ' =======================================================================
   ' PhidgetCurrentInput_getDataInterval()
   '     PhidgetReturnCode PhidgetCurrentInput_getDataInterval(
   '     PhidgetCurrentInputHandle  ch,
   '     uint32_t *dataInterval
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 250 ms
   '
   ' The DataInterval is the time that must elapse before the channel will
   ' fire another CurrentChange event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between CurrentChange
   ' events can also affected by the CurrentChangeTrigger.
    Public Declare Function PhCurInpGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getDataInterval" ( _
      ByVal handle&, _
      ByRef dataInterval&) ' Unsigned integer

   ' =======================================================================
   ' PhidgetCurrentInput_setDataInterval()
   '     PhidgetReturnCode PhidgetCurrentInput_setDataInterval(
   '     PhidgetCurrentInputHandle  ch,
   '     uint32_t  dataInterval
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Default: 250 ms

   ' The DataInterval is the time that must elapse before the channel will
   ' fire another CurrentChange event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between CurrentChange
   ' events can also affected by the CurrentChangeTrigger.
   '
    Public Declare Function PhCurInpSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_setDataInterval" ( _
      ByVal handle&, _
      ByVal dataInterval&) ' Unsigned integer

   ' =======================================================================

   ' PhidgetCurrentInput_getMinDataInterval()
   '     PhidgetReturnCode PhidgetCurrentInput_getMinDataInterval(
   '     PhidgetCurrentInputHandle  ch,
   '     uint32_t *minDataInterval
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     MinDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 20 ms
   '
   ' The minimum value that DataInterval can be set to.
   '
   Public Declare Function PhCurInpGetMinDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMinDataInterval" ( _
      ByVal handle&, _
      ByRef minDataInterval&) ' Unsigned integer

   ' =======================================================================
   ' PhidgetCurrentInput_getMaxDataInterval()
   '     PhidgetReturnCode PhidgetCurrentInput_getMaxDataInterval(
   '     PhidgetCurrentInputHandle  ch,
   '     uint32_t * maxDataInterval
   '     )
   ' The maximum value that DataInterval can be set to.
   ' Parameters:
   '     ch: The channel handle
   '     maxDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 60,000 ms
   '
   Public Declare Function PhCurInpGetMaxDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_getMaxDataInterval" ( _
      ByVal handle&, _
      ByRef maxDataInterval&) ' Unsigned integer


   ' =======================================================================
   ' EVENTS:
   ' =======================================================================
 
   ' =======================================================================
   ' PhidgetCurrentInput_setOnCurrentChangeHandler()
   '     PhidgetCurrentInputHandle ch,
   '     PhidgetCurrentInput_OnCurrentChangeCallback handler,
   '     void *ctx
   '     )
   ' Parameters:
   '     ch: The channel handle
   '     handler: The callback function
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the CurrentChange event occurs
   ' This function requires PhDigInpOnCurrentChange& callback procedure
   ' handlerPtr& must be set to AddressOf PhDigInpOnCurrentChange
   '
   ' Sample code for startup:
   '     stat& = PhOnDigInpCurrentChange&(handle&, _
   '             AddressOf PhOnDigInpCurrentChange, userData&)
   '
   Public Declare Function PhCurInpSetOnCurrentChange& _
      Lib "Phidget22.dll" _
      Alias "PhidgetCurrentInput_setOnCurrentChangeHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByRef userData&)
      
' ==========================================================================
' Event Callbacks, Subs, and Functions
' ==========================================================================

' ==========================================================================
   ' PhidgetCurrentInput_OnCurrentChangeCallback
   '     void CCONV onCurrentChange(
   '        PhidgetCurrentInputHandle  ch,
   '        void *ctx,
   '        double current
   '        )
   ' Parameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   '     current: The current value
   '
   '
 Public Static Function PhCurInpOnCurrentChange&( _
   ByVal handle&, _
   ByRef userData&, _
   ByVal current#)
   
   ' The most recent current value the channel has measured will be reported
   ' in this event, which occurs when the DataInterval has elapsed.
   ' If a CurrentChangeTrigger has been set to a non-zero value, the
   ' CurrentChange event will not occur until the current value has changed
   ' by at least the CurrentChangeTrigger value.
   '

   PhCurInpOnCurrentChange& = 0

End Function


'===========================================================================
' Event Callbacks, Subs, and Functions
'===========================================================================

' ==========================================================================
Public Static Sub InitPhCurInp(indx&, stat&)
   ' Startup Current Input
   
   ' Enter:    Dev(indx&)
   ' Return:   Dev(indx&).hndl&
   '           stat&  0=OK
   
   ' Create, configure, and open an Encoder channel.
   ' stat& = 0 if the program exits successfully
   
   ' ===================================
   ' Create an instance of a Phidget channel.  Once created the
   ' channel may need to be configured to match the physical
   ' device and channel you want to attach to, and then be
   ' opened by calling Phidget_open().  The channel handle must
   ' be released with PhidgetEncoder_delete().
   '
   ' stat& (Error Code) will always be 0 if no Errors
   '
   stat& = PhCurInpCreate&(Dev(indx&).hndl) ' Create new handle
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET SERIAL NUMBER
   stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
   If stat& <> 0 Then GoTo HandleErr

   ' SET ISHUB (0 if hub port is connected to a Phidget module)
   stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
   If stat& <> 0 Then GoTo HandleErr

   ' SET HUB PORT
   stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
   If stat& <> 0 Then GoTo HandleErr

   ' SET CHANNEL
   stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
   If stat& <> 0 Then GoTo HandleErr
   
' System crashes when using Callbacks! (and using camera DLL)
   
   ' Setup the handlers to be run when the device is plugged in,
   ' opened from software, unplugged, closed from software, or
   ' generates an error.
   ' SETUP ATTACH EVENT
   stat& = PhSetOnAttach&(Dev(indx&).hndl, _
               AddressOf PhOnAttach, _
               Dev(indx&).indx) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP DETACH EVENT
   stat& = PhSetOnDetach&(Dev(indx&).hndl, _
               AddressOf PhOnDetach, _
               Dev(indx&).indx) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ERROR EVENT
   ' Using common Error Handler for all phidget modules.
   stat& = PhSetOnError&(Dev(indx&).hndl, _
               AddressOf PhOnError, _
               Dev(indx&).indx) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ON CURRENT CHANGE EVENT
   stat& = PhCurInpSetOnCurrentChange&( _
               Dev(indx&).hndl, _
               AddressOf PhCurInpOnCurrentChange, _
               Dev(indx&).indx) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' OPEN CHANNEL
   'stat& = PhOpen(Dev(indx&).hndl)
   'If stat& <> 0 Then GoTo HandleErr
   
   ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
   stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, VintTimeOutMs&)
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET CURRENT CHANGE TRIGGER
   ' The channel will not issue a CurrentChange event until the current
   ' value has changed by the amount specified by the CurrentChangeTrigger.
   ' Setting the CurrentChangeTrigger to 0 will result in the channel firing
   ' events every DataInterval.  This is useful for applications that
   ' implement their own data filtering
   ' Set to 0 if using DATA INTERVAL
   stat& = PhCurInpSetCurrentChangeTrigger(Dev(indx&).hndl, 0&)
   If stat& <> 0 Then GoTo HandleErr

   ' SET DATA INTERVAL  (Default 250)
   '     dataInterval:  The time elapsed betweem position
   '                    change events in milliseconds

   ' The DataInterval is the time that must elapse before the channel will
   ' fire another CurrentChange event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between CurrentChange
   ' events can also affected by the CurrentChangeTrigger.
   ' Set to 0 if using CURRENT CHANGE TRIGGER
   stat& = PhCurInpSetDataInterval(Dev(indx&).hndl, 1000&)
   If stat& <> 0 Then GoTo HandleErr

   GoTo ExitSub
HandleErr:
   Form1.Label1 = "InitPhCurInp" & vbCrLf & GetBStrFromPtr$(stat&, True)

ExitSub:

End Sub
