Attribute VB_Name = "PhVintEnc"
Option Explicit
   
   '========================================================================
   ' Quadrature Encoder Phidget
   '========================================================================
   ' C ENC1000_0 - Quadrature Encoder Phidget Encoder API
   '
   ' Encoder wiring using Molex connector
   '  Black    Gnd      (Common to encoder)
   '  Green    Index    (Input from encoder) (not required)
   '  White    A        (Input from encoder)
   '  Red      +5V      (Power to 5Vdc encoder only)
   '  Brown    B        (Input from encoder)
   ' For 12 to 24Vdc use an encoder with open collector outputs.  Do not
   ' use the +5Vdc out.  Set mode 5 (open collector with 1k pull up) and
   ' power the encoder from an external 12/24Vdc source.  The Gnd wire
   ' should be connected to the encoder and the external source.
   '
   ' C++ code and comments are from Phidgets C++ API sample.
   '
   ' The Encoder class is used to read position data from quadrature
   ' encoders in order to track linear or rotary movement. If the device
   ' supports an index pin as a reference point, you can also access it
   ' through this class.
   
   Public EncPosition&

   '========================================================================
   ' Enumerations:
   '========================================================================
   
   '========================================================================
   ' Phidget_EncoderIOMode

   Const ENCODER_IO_MODE_PUSH_PULL = 1
   ' No additional pull-up or pull-down resistors will be applied
   ' to the input lines.
   
   Const ENCODER_IO_MODE_LINE_DRIVER_2K2 = 2
   ' 2.2k pull-down resistors will be applied to the input lines.
   
   Const ENCODER_IO_MODE_LINE_DRIVER_10K = 3
   ' 10k pull-down resistors will be applied to the input lines.
   
   Const ENCODER_IO_MODE_OPEN_COLLECTOR_2K2 = 4
   ' 2.2k pull-up resistors will be applied to the input lines.
   
   Const ENCODER_IO_MODE_OPEN_COLLECTOR_10K = 5
   ' 10k pull-up resistors will be applied to the input lines.

   '========================================================================
   ' Functions:
   '========================================================================
   
   '========================================================================
   ' PhidgetEncoder_create(
   '     PhidgetEncoderHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetEncoder_create() creates an instance of a Phidget channel.
   ' Once created the channel may need to be configured to match the
   ' physical device and channel index you want to attach to, and then
   ' be opened by calling Phidget_open().  The Phidget channel handle
   ' must be released with PhidgetEncoder_delete().
   '
   Public Declare Function PhEncCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_create" ( _
      ByRef phEncoderHandle&)

   '========================================================================
   ' PhidgetEncoder_delete(
   '     PhidgetEncoderHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetEncoder_delete() releases a Phidget channel handle created
   ' with PhidgetEncoder_create().  If the channel is open when
   ' PhidgetEncoder_delete() is called, it will be closed.
   '
   Public Declare Function PhEncDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_delete" ( _
      ByRef phEncoderHandle&)
   
   '========================================================================
   ' PhidgetEncoder_setEnabled(
   '     PhidgetEncoderHandle ch,
   '     int enabled
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     enabled: The enabled value
   ' Return: The Phidget return code
   ' Default: 1
   '
   ' Set enabled state of the encoder.
   '
   Public Declare Function PhEncSetEnabled& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setEnabled" ( _
      ByVal phEncoderHandle&, _
      ByVal enab&)
   
   '========================================================================
   ' PhidgetEncoder_getEnabled(
   '     PhidgetEncoderHandle ch,
   '     int *enabled
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     enab&: The enabled value
   ' Return: The Phidget return code
   ' Default: 1
   '
   ' Get enabled state of the encoder.
   '
   Public Declare Function PhEncGetEnabled& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getEnabled" ( _
      ByVal phEncoderHandle&, _
      ByRef enab&)
   
   '========================================================================
   ' PhidgetEncoder_getDataInterval(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *dataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 250 ms
   '
   ' The dataInterval is the time that must elapse before the channel
   ' will fire another PositionChange event.  The data interval is bounded
   ' by MinDataInterval and MaxDataInterval.  The timing  between
   ' PositionChange events can also affected by the PositionChangeTrigger.
   '
   Public Declare Function PhEncGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getDataInterval" ( _
      ByVal phEncoderHandle&, _
      ByRef dataInterval&) ' Unsigned integer

   '========================================================================
   ' PhidgetEncoder_setDataInterval(
   '     PhidgetEncoderHandle ch,
   '     uint32_t dataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 250 ms
   '
   ' The dataInterval is the time that must elapse before the channel
   ' will fire another PositionChange event.  The data interval is
   ' bounded by MinDataInterval and MaxDataInterval.  The timing between
   ' PositionChange events can also affected by the PositionChangeTrigger.
   '
   Public Declare Function PhEncSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setDataInterval" ( _
      ByVal phEncoderHandle&, _
      ByVal dataInterval&) ' Unsigned integer
   
   '========================================================================
   ' PhidgetEncoder_getMinDataInterval(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *minDataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 20 ms
   '
   ' The minimum value that DataInterval can be set to.
   '
   Public Declare Function PhEncGetMinDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getMinDataInterval" ( _
      ByVal phEncoderHandle&, _
      ByRef minDataInterval&) ' Unsigned integer
   
   '========================================================================
   ' PhidgetEncoder_getMaxDataInterval(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *maxDataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 1,000 ms
   '
   ' The maximum value that DataInterval can be set to.
   '
   Public Declare Function PhEncGetMaxDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getMaxDataInterval" ( _
      ByVal phEncoderHandle&, _
      ByRef maxDataInterval&) ' Unsigned integer

   '========================================================================
   ' PhidgetEncoder_getIndexPosition(
   '     PhidgetEncoderHandle ch,
   '     int64_t *indexPosition
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     indexPosition: The index position value
   ' Return: The Phidget return code
   '     Constant: 0
   '
   ' The most recent position of the index channel calculated by the
   ' Phidgets library.  The index channel will usually pulse once per
   ' rotation.  Setting the encoder position will move the index position
   ' the same amount so their relative position stays the same.
   ' Index position is tracked locally as the last position at which
   ' the index was triggered. Setting position will only affect the
   ' local copy of the index position value. This means that index
   ' positions seen by multiple network applications may not agree.
   '
   Public Declare Function PhEncGetIndexPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getIndexPosition" ( _
      ByVal phEncoderHandle&, _
      ByRef indexPosition@) ' 64bit Currency (requires special handling)
   
   '========================================================================
   ' PhidgetEncoder_getIOMode(
   '     PhidgetEncoderHandle ch,
   '     Phidget_EncoderIOMode *ioMode
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     ioMode: The IO mode value.
   ' Return: The Phidget return code
   ' Default: ENCODER_IO_MODE_PUSH_PULL
   '
   ' The encoder interface mode. Match the mode to the type of encoder
   ' you have attached.  It is recommended to only change this when the
   ' encoder is disabled in order to avoid unexpected results.
   '
   Public Declare Function PhEncGetIOMode& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getIOMode" ( _
      ByVal phEncoderHandle&, _
      ByRef ioMode&)
    
   '========================================================================
   ' PhidgetEncoder_setIOMode(
   '     PhidgetEncoderHandle ch,
   '     Phidget_EncoderIOMode ioMode
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     ioMode: The IO mode value.
   ' Return: The Phidget return code
   ' Default: ENCODER_IO_MODE_PUSH_PULL
   '
   ' The encoder interface mode. Match the mode to the type of encoder
   ' you have attached.  It is recommended to only change this when the
   ' encoder is disabled in order to avoid unexpected results.
   '
   Public Declare Function PhEncSetIOMode& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setIOMode" ( _
      ByVal phEncoderHandle&, _
      ByVal ioMode&)

   '========================================================================
   ' PhidgetEncoder_getPosition(
   '     PhidgetEncoderHandle ch,
   '     int64_t *position
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     position: The position value
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The most recent position value calculated by the Phidgets library.
   ' Position counts quadrature edges within a quadrature cycle. This
   ' means there are four counts per full quadrature cycle.  Position
   ' is tracked locally as the total position change from the time the
   ' channel is opened. Setting position will only affect the local
   ' copy of the position value. This means that positions seen by
   ' multiple network applications may not agree.
   '
   Public Declare Function PhEncGetPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getPosition" ( _
      ByVal phEncoderHandle&, _
      ByRef position@) ' Currency64bit (actual value is position@ * 10000)

   '========================================================================
   ' PhidgetEncoder_setPosition(
   '     PhidgetEncoderHandle ch,
   '     int64_t position
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     position: The position value
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The most recent position value calculated by the Phidgets library.
   ' Position counts quadrature edges within a quadrature cycle. This
   ' means there are four counts per full quadrature cycle.  Position
   ' is tracked locally as the total position change from the time
   ' the channel is opened. Setting position will only affect the local
   ' copy of the position value. This means that positions seen by
   ' multiple network applications may not agree.
   '
   Public Declare Function PhEncSetPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setPosition" ( _
      ByVal phEncoderHandle&, _
      ByVal position@) ' Currency64bit (set to position@ / 10000)
   
   '========================================================================
   ' PhidgetEncoder_getPositionChangeTrigger(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *positionChangeTrigger
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     positionChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The channel will not issue a PositionChange event until the position
   ' value has changed by the amount specified by the PositionChangeTrigger.
   ' Setting the PositionChangeTrigger to 0 will result in the channel
   ' firing events every DataInterval.  This is useful for applications
   ' that implement their own data filtering.
   '
   Public Declare Function PhEncGetPositionChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getPositionChangeTrigger" ( _
      ByVal phEncoderHandle&, _
      ByRef positionChangeTrigger&) ' Unsigned integer
   
   '========================================================================
   ' PhidgetEncoder_setPositionChangeTrigger(
   '     PhidgetEncoderHandle ch,
   '     uint32_t positionChangeTrigger
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     positionChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The channel will not issue a PositionChange event until the position
   ' value has changed by the amount specified by the PositionChangeTrigger.
   ' Setting the PositionChangeTrigger to 0 will result in the channel
   ' firing events every DataInterval.  This is useful for applications
   ' that implement their own data filtering.
   '
   Public Declare Function PhEncSetPositionChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setPositionChangeTrigger" ( _
      ByVal phEncoderHandle&, _
      ByVal positionChangeTrigger&) ' Unsigned integer
   
   '========================================================================
   ' PhidgetEncoder_getMinPositionChangeTrigger(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *minPositionChangeTrigger
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minPositionChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 0
   '
   ' The minimum value that PositionChangeTrigger can be set to.
   '
   Public Declare Function PhEncGetMinPositionChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getMinPositionChangeTrigger" ( _
      ByVal phEncoderHandle&, _
      ByRef minPositionChangeTrigger&) ' Unsigned integer
   
   '========================================================================
   ' PhidgetEncoder_getMaxPositionChangeTrigger(
   '     PhidgetEncoderHandle ch,
   '     uint32_t *maxPositionChangeTrigger
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxPositionChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 60,000,000
   '
   ' The maximum value that PositionChangeTrigger can be set to.
   '
   Public Declare Function PhEncGetMaxPositionChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_getMaxPositionChangeTrigger" ( _
      ByVal phEncoderHandle&, _
      ByRef maxPositionChangeTrigger&) ' Unsigned integer
   
   
   '========================================================================
   ' Events:
   '========================================================================
   
   '========================================================================
   ' PhidgetEncoder_setOnPositionChangeHandler(
   '     PhidgetEncoderHandle ch,
   '     PhidgetEncoder_OnPositionChangeCallback handler,
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     handler: The callback function
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the PositionChange
   ' event occurs.
   '
   ' Note PhOnPositionChangeEventPtr& is "AddressOf" procedure to
   '      be run when DataInterval has elapsed.
   ' handlerPtr& can be set to AddressOf PhEncOnPositionChange
   '
   Public Declare Function PhEncSetOnPositionChangeHandler& _
      Lib "Phidget22.dll" _
      Alias "PhidgetEncoder_setOnPositionChangeHandler" ( _
      ByVal phEncoderHandle&, _
      ByVal handlerPtr&, _
      ByRef userPtr&)
    

'===========================================================================
' Event Callbacks, Subs, and Functions
'===========================================================================

'===========================================================================
' onPositionChangeHandler(
'     PhidgetEncoderHandle ch,
'     void *ctx,
'     int positionChange,
'     double timeChange,
'     int indexTriggered
' )
Public Static Function PhEncOnPositionChange&( _
   ByVal phEncoderHandle&, _
   ByRef userPtr&, _
   ByVal posChange&, _
   ByVal updTime#, _
   ByVal indexTriggered&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   EncPosition& = EncPosition& + posChange&
   
   ' VB Scroll Bar limit is -32768 to 32767
   'If EncPosition& > 32767 Then EncPosition& = 32767
   'If EncPosition& < -32768 Then EncPosition& = -32768
   
   ' Don't allow overflow - Rollover at 3FFFFFFF
   'If EncPosition& < 0 Then EncPosition& = EncPosition& + 268435456
   'If EncPosition& > 268435455 Then EncPosition& = EncPosition& -268435456
   '
   If EncPosition& < 0 Then
      EncPosition& = 0
      'stat& = PhEncSetPosition&(Dev(indx&).hndl, 0@)
   End If

   ' Limit high val to 100000
   ' Not required if OK to rollover at 268435456 to -268435456
   If EncPosition& > 100000 Then EncPosition& = 100000
   ' ===================================

   PhEncOnPositionChange& = 0
   
End Function


'===========================================================================
' PHIDGET ENC1000 ENCODER STARTUP
'===========================================================================

'===========================================================================
Public Static Sub InitPhEncoder(indx&, stat&)
   ' Startup Encoder
   ' Enter with indx& of Encoder from PhCommon22.bas
   ' System crashes when using Callbacks! (and using camera DLL)
   
   ' Enter:    Dev(indx&)
   ' Return:   Dev(indx&).hndl&
   '           stat&  0=OK
   
   ' Create, configure, and open an Encoder channel.
   ' stat& = 0 if the program exits successfully
   
   Dim attached&
   
   On Error Resume Next
   stat& = 0
   
   If Dev(indx&).hndl = 0 Then
      ' ===================================
      ' Create an instance of a Phidget channel.  Once created the
      ' channel may need to be configured to match the physical
      ' device and channel you want to attach to, and then be
      ' opened by calling Phidget_open().  The channel handle must
      ' be released with PhidgetEncoder_delete().
      '
      ' stat& (Error Code) will always be 0 if no Errors
      '
      stat& = PhEncCreate&(Dev(indx&).hndl) ' Create new handle
      If stat& <> 0 Then GoTo HandleErr
      
      ' SET SERIAL NUMBER
      stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
      If stat& <> 0 Then GoTo HandleErr
   
      ' SET ISHUB (0 if hub port is connected to a Phidget module)
      stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
      If stat& <> 0 Then GoTo HandleErr
   
      ' SET HUB PORT
      stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
      If stat& <> 0 Then GoTo HandleErr
   
      ' SET CHANNEL
      stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
      If stat& <> 0 Then GoTo HandleErr
      
      ' Setup the handlers to be run when the device is plugged in,
      ' opened from software, unplugged, closed from software, or
      ' generates an error.
      ' SETUP ATTACH EVENT
      stat& = PhSetOnAttach&(Dev(indx&).hndl, _
                  AddressOf PhOnAttach, _
                  Dev(indx&).indx) ' returned with event for this handle
      If stat& <> 0 Then GoTo HandleErr
   
      ' SETUP DETACH EVENT
      stat& = PhSetOnDetach&(Dev(indx&).hndl, _
                  AddressOf PhOnDetach, _
                  Dev(indx&).indx) ' returned with event for this handle
      If stat& <> 0 Then GoTo HandleErr
   
      ' SETUP ERROR EVENT
      ' Using common Error Handler for all phidget modules.
      stat& = PhSetOnError&(Dev(indx&).hndl, _
                  AddressOf PhOnError, _
                  Dev(indx&).indx) ' returned with event for this handle
      If stat& <> 0 Then GoTo HandleErr
   
      ' SETUP ON POSITION CHANGE EVENT
      stat& = PhEncSetOnPositionChangeHandler&( _
                  Dev(indx&).hndl, _
                  AddressOf PhEncOnPositionChange, _
                  Dev(indx&).indx) ' returned with event for this handle
      If stat& <> 0 Then GoTo HandleErr
   
   End If
   
   stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
   If attached& <> 1 Then
      ' OPEN CHANNEL
      'stat& = PhOpen(Dev(indx&).hndl)
      'If stat& <> 0 Then GoTo HandleErr
         
      ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
      stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, VintTimeOutMs&)
      If stat& <> 0 Then stat& = &H34: GoTo HandleErr
      
      ' ENABLE CHANNEL (Default ON) (Must OPEN first)
      stat& = PhEncSetEnabled&(Dev(indx&).hndl, 1&)
      If stat& <> 0 Then GoTo HandleErr
   End If
   
   stat& = PhEncSetIOMode(Dev(indx&).hndl, 5) 'ENCODER_IO_MODE_PUSH_PULL)
   If stat& <> 0 Then GoTo HandleErr

   ' SET POSITION CHANGE TRIGGER
   ' The channel will not issue a PositionChange event until the position
   ' value has changed by the amount specified by the PositionChangeTrigger.
   ' Setting the PositionChangeTrigger to 0 will result in the channel
   ' firing events every DataInterval.  This is useful for applications
   ' that implement their own data filtering.
   stat& = PhEncSetPositionChangeTrigger(Dev(indx&).hndl, 0&)
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET DATA INTERVAL (Default 250)
   ' The DataInterval is the time that must elapse before the channel
   ' will fire another PositionChange event.  The data interval is
   ' bounded by MinDataInterval and MaxDataInterval.  The timing between
   ' PositionChange events can also be affected by the PositionChangeTrigger.
   ' Set to 0 if using POSITION CHANGE TRIGGER
   stat& = PhEncSetDataInterval(Dev(indx&).hndl, 20&)
   If stat& <> 0 Then GoTo HandleErr
   
   Exit Sub

HandleErr:
   'Call IoDisplay.DisplayMsg("A004  InitPhEncoder" & vbCrLf _
                             & GetBStrFromPtr$(stat&, True), 100)
   Form1.Label1 = "A004  InitPhEncoder" & vbCrLf _
                    & GetBStrFromPtr$(stat&, True)
End Sub
   

