Attribute VB_Name = "PhVintSetup"
Option Explicit
  
   ' Typical C++ data type for VINT (from Phidgets C++ example)
   '
   '   Public Type ChInfo
   '      deviceSerialNumber As Long
   '      hubPort As Long
   '      channel As Long
   '      isHub As Long  ' Is this a Hub Port device
   '      isVINT As Long ' Is this a VINT device
   '      netInf As NetInfo
   '   End Type
   '
   '   Public Type NetInfo
   '      isRemote As Long
   '      serverDiscovery As Long
   '      hostName As String
   '      netPort As Long
   '      password As String
   '   End Type
   
   ' Note: Using modified ChInfo in this program
   '       Not using NetInfo in this program
   
   ' =============================
   ' Create a TYPE for the data required for each I/O point
   Public Type ChInfo
      ' Values entered by user
      devName As String    ' Device Name (Any String)
      devClass As String   ' DigOut, DigInp, Encoder, Stepper
      hubSn As Long        ' Nbr on back of Hub
      isHub As Long        ' 1 if using Hub Port as I/O point
      port As Long         ' Hub Port number
      ch As Long           ' Module channel number (0 if Hub)
      
      ' The following values are updated by the program
      hndl As Long         ' Handle is created with 'Create'
      indx As Long         ' Added so OnAttach and OnDetatch can return
      attach As Boolean    ' with index of device.
   End Type
      
   Public PhVintHubSn0Sp&  ' Phidgets HUB serial number
   Public PhBusy As Boolean
      
   ' Make an array large enough for all I/O points
   Public Dev(63) As ChInfo ' 64 devices
   Public Const VintTimeOutMs& = 500
   
   Public Stepper01Index&
   Public Stepper02Index&
   Public EncoderIndex&
   Public RelOut00Index&
   Public RelOut01Index&
   Public RelOut02Index&
   Public RelOut03Index&
   Public LcdDisplayIndex&
   Public DigOut01Index&
   
   Public CurInpIndex&
   
   
   ' Typical C++ data type for VINT (from Phidgets C++ example)
   '
   '   Public Type NetInfo
   '      isRemote As Long
   '      serverDiscovery As Long
   '      hostName As String
   '      netPort As Long
   '      password As String
   '   End Type
   '
   '   Public Type ChInfo
   '      deviceSerialNumber As Long
   '      hubPort As Long
   '      channel As Long
   '      isHub As Long  ' Is this a Hub Port device
   '      isVINT As Long ' Is this a VINT device
   '      netInf As NetInfo
   '   End Type

' ===========================================================================
Public Static Sub SetupPhidgetsDevices(stat&)
   ' User setup data
   ' Enter SN#, isHub, port, and ch for each I/O point
   
   Dim indx&
   ' Each I/O point is given an index number.  This is passed to the
   ' Phiget22 DLL (as userData) when the callbacks (events) are set up.
   ' When an event occurs, the handle and index (userData) are sent
   ' back to the callback routine (eg. OnError) and used as as the
   ' ChInfo() index to reference which I/O point the event belongs to.
   
   ' =================================================
   ' Manually set device indexes here
   Stepper01Index& = 0     ' Hub 0, Port 0, Channel 0
   Stepper02Index& = 1     ' Hub 0, Port 1, Channel 0
   EncoderIndex& = 2       ' Hub 0, Port 2, Channel 0
   RelOut00Index& = 3      ' Hub 0, Port 3, Channel 0
   RelOut01Index& = 4      ' Hub 0, Port 3, Channel 1
   RelOut02Index& = 5      ' Hub 0, Port 3, Channel 2
   RelOut03Index& = 6      ' Hub 0, Port 3, Channel 3
   DigOut01Index& = 7      ' Hub 0, Port 4
   LcdDisplayIndex& = 8    ' Hub 0, Port 5, Channel 0
   CurInpIndex& = 9        ' Hub 1, Port 0, Channel 0
   
   ' =================================================
   ' Setup Hub 0, Port 0, Channel 0 for STC1001 Stepper 0
   indx& = Stepper01Index&
   Dev(indx&).devName = "Stepper 000"  ' Enter name of device
   Dev(indx&).devClass = "Stepper"     ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' Enter SN# for YOUR hub
   Dev(indx&).port = 0                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhStepper(indx&, stat&)    ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 1, Channel 0 for STC1001 Stepper 1
   ' Setup VINT Hub hubPort for STC1001 Stepper 1
   indx& = Stepper02Index&
   Dev(indx&).devName = "Stepper 010"  ' Enter name of device
   Dev(indx&).devClass = "Stepper"     ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' Enter SN# for YOUR hub
   Dev(indx&).port = 1                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhStepper(indx&, stat&)    ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 2, Channel 0 for ENC1000 Encoder
   indx& = EncoderIndex&
   Dev(indx&).devName = "Encoder 020"  ' Enter name of device
   Dev(indx&).devClass = "Encoder"     ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' Enter SN# for YOUR hub
   Dev(indx&).port = 2                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhEncoder(indx&, stat&)    ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 3, Channel 0 for Relay Output 0
   indx& = RelOut00Index&
   Dev(indx&).devName = "RelOut 030"   ' Enter name of device
   Dev(indx&).devClass = "DigOut"      ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' SN# for hub
   Dev(indx&).port = 3                 ' Hub port number (0-5)
   Dev(indx&).ch = 0                   ' Module channel or 0 for Hub
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhDigOut(indx&, stat&)     ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 3, Channel 1 for Relay Output 1
   indx& = RelOut01Index&
   Dev(indx&).devName = "RelOut 031"   ' Enter name of device
   Dev(indx&).devClass = "DigOut"      ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' SN# for hub
   Dev(indx&).port = 3                 ' Hub port number (0-5)
   Dev(indx&).ch = 1                   ' Module channel or 0 for Hub
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhDigOut(indx&, stat&)     ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 3, Channel 2 for Relay Output 2
   indx& = RelOut02Index&
   Dev(indx&).devName = "RelOut 032"   ' Enter name of device
   Dev(indx&).devClass = "DigOut"      ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' SN# for hub
   Dev(indx&).port = 3                 ' Hub port number (0-5)
   Dev(indx&).ch = 2                   ' Module channel or 0 for Hub
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhDigOut(indx&, stat&)     ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 3, Channel 3 for Relay Output 3
   indx& = RelOut03Index&
   Dev(indx&).devName = "RelOut 033"   ' Enter name of device
   Dev(indx&).devClass = "DigOut"      ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' SN# for hub
   Dev(indx&).port = 3                 ' Hub port number (0-5)
   Dev(indx&).ch = 3                   ' Module channel or 0 for Hub
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhDigOut(indx&, stat&)     ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 4, Channel 0 for Local Output 1
   indx& = DigOut01Index&
   Dev(indx&).devName = "DigOut 040"   ' Enter name of device
   Dev(indx&).devClass = "DigOut"      ' Enter class of device
   Dev(indx&).isHub = 1                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' Serial number for MY hub
   Dev(indx&).port = 4                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhDigOut(indx&, stat&)     ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 0, Port 5, Channel 0 for LCD Display Module
   indx& = LcdDisplayIndex&
   Dev(indx&).devName = "Lcd 050"      ' Enter name of device
   Dev(indx&).devClass = "Lcd"         ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 540040           ' Enter SN# for YOUR hub
   Dev(indx&).port = 5                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhLcd(indx&, stat&)        ' Create Dev(indx&).hndl
   
   ' =================================================
   ' Setup Hub 1, Port 0, Channel 0 for VCP1100 Current Input
   indx& = CurInpIndex&
   Dev(indx&).devName = "CurInp 100"   ' Enter name of device
   Dev(indx&).devClass = "CurInp"      ' Enter class of device
   Dev(indx&).isHub = 0                ' Located on hub (0=No 1=Yes)
   Dev(indx&).hubSn = 527526           ' Enter SN# for YOUR hub
   Dev(indx&).port = 0                 ' Hub port (0-5)
   Dev(indx&).ch = 0                   ' Channel (Hub is always 0)
   Dev(indx&).indx = indx&             ' For VB
   Call InitPhCurInp(indx&, stat&)     ' Create Dev(indx&).hndl
   
End Sub

'===========================================================================
Public Static Sub CloseAllPhidgets()
   ' Close (if attached) and Delete all Created channels.
   ' Devices (channels) are created using their "class".  They
   ' are also be deleted by class.  The devClass is set during
   ' entry of setup pramaters.
  
   Dim indx&
   Dim stat&
   Dim attached&
   
   On Error Resume Next
   
   For indx& = 0 To 63 ' Skips devices not used
      If Dev(indx&).hndl <> 0 Then
         Select Case Dev(indx&).devClass
            Case "DigOut"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhDigOutDelete&(Dev(indx&).hndl)
            Case "DigInp"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhDigInpDelete&(Dev(indx&).hndl)
            Case "Encoder"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhEncDelete&(Dev(indx&).hndl)
            Case "Stepper"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhStepDelete&(Dev(indx&).hndl)
            Case "CurInp"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhCurInpDelete&(Dev(indx&).hndl)
            Case "Lcd"
               stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
               If attached& = 1 Then stat& = PhClose(Dev(indx&).hndl)
               stat& = PhLcdDelete&(Dev(indx&).hndl)
         End Select ' Dev(indx&).devClass
         Sleep 50
     End If
   Next indx&

   Sleep 500 ' Allow some time to close before ending (avoid crash)

End Sub
   
   

