Attribute VB_Name = "PhVintTemp"
Option Explicit
   '
   '========================================================================
   ' Phidget Temperature Sensor
   '========================================================================
   '
   ' NOTE: Most comments and sample code are from Phidgets C API.
   '
   ' The Temperature Sensor class gathers data from the temperature
   ' sensor on a Phidget board.  This includes on-board ambient
   ' temperature sensors, connected thermocouples or platinum RTDs,
   ' and IR temperature sensors.  This class is also used to measure
   ' the temperature on some high-power Phidget boards such as motor
   ' controllers for safety reasons.
   '
   ' If you're using a simple 0-5V sensor that does not have its own
   ' firmware, use the VoltageInput or VoltageRatioInput class instead,
   ' as specified for your device.
  
  
   '========================================================================
   ' Functions:
   '========================================================================
   '
   '========================================================================
   'Public Declare Function PhTempCreate Lib "Phidget22.dll" Alias "PhidgetTemperatureSensor_create" (ByRef phTemperatureHandle As Long) As Long
   Public Declare Function PhTempCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_create" ( _
      ByRef phTemperatureHandle&)

   ' PhidgetTemperatureSensor_create(
   '    PhidgetTemperatureSensorHandle *ch)
   ' )
   ' Parameters:
   '    ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetTemperatureSensor_create() creates an instance of a Phidget
   ' channel.  Once created the channel may need to be configured to match
   ' the physical device and channel index you want to attach to, and then
   ' be opened by calling Phidget_open().
   '
   ' The Phidget channel handle must be released with
   ' PhidgetTemperatureSensor_delete().
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle *ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    // Do work...
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_delete" ( _
      ByRef phTemperatureHandle&)

   ' PhidgetTemperatureSensor_delete(
   '    PhidgetTemperatureSensorHandle *ch
   ' )
   ' Parameters:
   '    ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetTemperatureSensor_delete() releases a Phidget
   ' channel handle created with PhidgetTemperatureSensor_create().
   ' If the channel is open when PhidgetTemperatureSensor_delete()
   ' is called, it will be closed.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle *ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    // Do work...
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getDataInterval" ( _
      ByVal phTemperatureHandle&, _
      ByRef dataInterval&) ' Unsigned integer
   
   ' PhidgetTemperatureSensor_getDataInterval(
   '    PhidgetTemperatureSensorHandle ch,
   '    uint32_t *dataInterval
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 500 ms
   '
   ' The dataInterval is the time that must elapse before the channel will
   ' fire another TemperatureChange event.
   ' The data interval is bounded by MinDataInterval and MaxDataInterval.
   ' The timing between TemperatureChange events is also affected by
   ' the TemperatureChangeTrigger.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    uint32_t dataInterval;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getDataInterval(ch, &dataInterval);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_setDataInterval" ( _
      ByVal phTemperatureHandle&, _
      ByVal dataInterval&) ' Unsigned integer
   
   ' PhidgetTemperatureSensor_setDataInterval(
   '    PhidgetTemperatureSensorHandle ch,
   '    uint32_t DataInterval
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    DataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 500 ms
   '
   ' The DataInterval is the time that must elapse before the channel will
   ' fire another TemperatureChange event.
   ' The data interval is bounded by MinDataInterval and MaxDataInterval.
   ' The timing between TemperatureChange events is also affected by
   ' the TemperatureChangeTrigger.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_setDataInterval(ch, 1000);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMinDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMinDataInterval" ( _
      ByVal phTemperatureHandle&, _
      ByRef minDataInterval&) ' Unsigned integer
   
   ' PhidgetTemperatureSensor_getMinDataInterval(
   '    PhidgetTemperatureSensorHandle ch,
   '    uint32_t *minDataInterval
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    minDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 500 ms
   '
   ' The minimum value that dataInterval can be set to.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    uint32_t minDataInterval;
   '    PhidgetTemperatureSensor_create(&ch);
   '
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMinDataInterval(ch, &minDataInterval);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMaxDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMaxDataInterval" ( _
      ByVal phTemperatureHandle&, _
      ByRef maxDataInterval&) ' Unsigned integer
   
   ' PhidgetTemperatureSensor_getMaxDataInterval(
   '    PhidgetTemperatureSensorHandle ch,
   '    uint32_t *maxDataInterval
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    maxDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 60,000 ms
   '
   ' The maximum value that DataInterval can be set to.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    uint32_t maxDataInterval;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMaxDataInterval(ch, &maxDataInterval);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetTemperature& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getTemperature" ( _
      ByVal phTemperatureHandle&, _
      ByRef temperature#)
   
   ' PhidgetTemperatureSensor_getTemperature(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *temperature
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    temperature: The temperature value
   ' Return: The Phidget return code
   ' Unit: degrees celsius (C)
   '
   ' The most recent temperature value that the channel has reported.
   '
   '     This value will always be between MinTemperature and
   '     MaxTemperature.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double temperature;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getTemperature(ch, &temperature);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMinTemperature& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMinTemperature" ( _
      ByVal phTemperatureHandle&, _
      ByRef minTemperature#)
   
   ' PhidgetTemperatureSensor_getMinTemperature(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *minTemperature
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    MinTemperature: The temperature value
   ' Return: The Phidget return code
   ' Constant: -40.0 C
   '
   ' The minimum value the TemperatureChange event will report.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double minTemperature;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMinTemperature(ch, &minTemperature);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMaxTemperature& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMaxTemperature" ( _
      ByVal phTemperatureHandle&, _
      ByRef maxTemperature#)
   
   ' PhidgetTemperatureSensor_getMaxTemperature()
   ' Constant: 85.0 C
   '    PhidgetReturnCode
   ' PhidgetTemperatureSensor_getMaxTemperature(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *maxTemperature
   '    )
   '
   ' The maximum value the TemperatureChange event will report.
   '
   ' Parameters:
   '    ch: The channel handle
   '    MaxTemperature: The temperature value
   ' Return: The Phidget return code
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double maxTemperature;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMaxTemperature(ch, &maxTemperature);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetTemperatureChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getTemperatureChangeTrigger" ( _
      ByVal phTemperatureHandle&, _
      ByRef temperatureChangeTrigger#)
   
   ' PhidgetTemperatureSensor_getTemperatureChangeTrigger(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *temperatureChangeTrigger
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    temperatureChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0.0 C
   '
   ' The channel will not issue a TemperatureChange event until the
   ' temperature value has changed by the amount specified by the
   ' TemperatureChangeTrigger.
   ' Setting the TemperatureChangeTrigger to 0 will result in
   ' the channel firing events every DataInterval.  This is useful
   ' for applications that implement their own data filtering
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double temperatureChangeTrigger;
   '    PhidgetTemperatureSensor_create(&ch);
   '
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getTemperatureChangeTrigger(ch, &temperatureChangeTrigger);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempSetTempChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_setTemperatureChangeTrigger" ( _
      ByVal phTemperatureHandle&, _
      ByVal temperatureChangeTrigger#)
   
   ' PhidgetTemperatureSensor_setTemperatureChangeTrigger(
   '    phidgetTemperatureSensorHandle ch,
   '    double temperatureChangeTrigger
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    temperatureChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Default: 0.0 C
   '
   ' The channel will not issue a TemperatureChange event until the
   ' temperature value has changed by the amount specified by the
   ' TemperatureChangeTrigger.
   ' Setting the TemperatureChangeTrigger to 0 will result in
   ' the channel firing events every DataInterval.  This is useful
   ' for applications that implement their own data filtering
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_setTemperatureChangeTrigger(ch, 0);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMinTemperatureChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMinTemperatureChangeTrigger" ( _
      ByVal phTemperatureHandle&, _
      ByRef minTemperatureChangeTrigger#)
   
   ' PhidgetTemperatureSensor_getMinTemperatureChangeTrigger(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *minTemperatureChangeTrigger
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    MinTemperatureChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 0.0 C
   '
   ' The minimum value that TemperatureChangeTrigger can be set to.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double minTemperatureChangeTrigger;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMinTemperatureChangeTrigger(ch, &minTemperatureChangeTrigger);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   Public Declare Function PhTempGetMaxTemperatureChangeTrigger& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_getMaxTemperatureChangeTrigger" ( _
      ByVal phTemperatureHandle&, _
      ByRef maxTemperatureChangeTrigger#)
   
   ' PhidgetTemperatureSensor_getMaxTemperatureChangeTrigger(
   '    PhidgetTemperatureSensorHandle ch,
   '    double *maxTemperatureChangeTrigger
   '    )
   '
   ' The maximum value that TemperatureChangeTrigger can be set to.
   '
   ' Parameters:
   '    ch: The channel handle
   '    MaxTemperatureChangeTrigger: The change trigger value
   ' Return: The Phidget return code
   ' Constant: 125.0 C
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    double maxTemperatureChangeTrigger;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = Phidget_openWaitForAttachment((PhidgetHandle)ch, PHIDGET_TIMEOUT_DEFAULT);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = PhidgetTemperatureSensor_getMaxTemperatureChangeTrigger(ch, &maxTemperatureChangeTrigger);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
   '========================================================================
   ' Events:
   '========================================================================
   
   '========================================================================
   Public Declare Function PhTempSetOnTemperatureChangeHandler& _
      Lib "Phidget22.dll" _
      Alias "PhidgetTemperatureSensor_setOnTemperatureChangeHandler" ( _
      ByVal phTemperatureHandle&, _
      ByVal handlerPtr&, _
      ByRef userPtr&)
   '
   ' Note: handlerPtr& is AddressOf PhVintTempOnTemperatureChange& when called
   '
   ' PhidgetTemperatureSensor_setOnTemperatureChangeHandler(
   '    PhidgetTemperatureSensorHandle ch,
   '    PhidgetTemperatureSensor_OnTemperatureChangeCallback handler,
   '    void *ctx
   '    )
   ' Parameters:
   '    ch: The channel handle
   '    handler: The callback function
   '    ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the TemperatureChange event occurs
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' void CCONV temperatureChangeHandler(
   '  PhidgetTemperatureSensorHandle ch,
   '  void *ctx,
   '  double temperature) {
   '    // Work with event data...
   ' }
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = PhidgetTemperatureSensor_setOnTemperatureChangeHandler(ch, temperatureChangeHandler, NULL);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = Phidget_open((PhidgetHandle)ch);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    while (1) {
   '       ; // Do work, wait for events, etc.
   '    }
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
'===========================================================================
' Event Callbacks, Subs, and Functions
'===========================================================================

'============================================================================
Public Static Function PhVintTempOnTemperatureChange&( _
   ByVal phTemperatureHandle&, _
   ByRef userPtr&, _
   ByVal temperature#)
   
   ' ========== User code HERE =========
   ' some code
   'PhVintTempTemperature# = temperature# ' PhTempTemperature# is Global Variable
   ' ===================================

   PhVintTempOnTemperatureChange& = 0 ' Return 0 = OK
   
   '========================================================================
   ' PhidgetTemperatureSensor_OnTemperatureChangeCallback
   '    void CCONV onTemperatureChange(
   '    PhidgetTemperatureSensorHandle ch,
   '    void *ctx,
   '    double temperature
   '    )
   ' ch: The channel handle
   ' ctx: Pointer to the user context
   ' temperature: The temperature
   '
   ' The most recent temperature value the channel has measured
   ' will be reported in this event, which occurs when the
   ' dataInterval has elapsed.
   '
   ' If a TemperatureChangeTrigger has been set to a non-zero
   ' value, the TemperatureChange event will not occur until
   ' the temperature has changed by at least the
   ' TemperatureChangeTrigger value.
   '
   ' C Code Example:
   ' #include <stdlib.h>
   ' #include "phidget22.h"
   ' void CCONV temperatureChangeHandler(PhidgetTemperatureSensorHandle ch, void *ctx, double temperature) {
   '    // Work with event data...
   ' }
   ' int main() {
   '    PhidgetReturnCode res;
   '    PhidgetTemperatureSensorHandle ch;
   '    PhidgetTemperatureSensor_create(&ch);
   '    res = PhidgetTemperatureSensor_setOnTemperatureChangeHandler(ch, temperatureChangeHandler, NULL);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    res = Phidget_open((PhidgetHandle)ch);
   '    if (res != EPHIDGET_OK)
   '       return 1; // Exit in error
   '    while (1) {
   '       ; // Do work, wait for events, etc.
   '    }
   '    PhidgetTemperatureSensor_delete(&ch);
   '    return 0;
   ' }
   
End Function
   
'===========================================================================
Public Static Sub InitPhVintTemp(indx&, stat&)
   ' Startup Temperature Sensor
   ' Enter with indx& of TempSensor from PhCommon22.bas
   
   ' Note: My system crashes when using Callbacks and 3rd party camera DLL.
   
   ' Enter:    Dev(indx&)
   ' Return:   Dev(indx&).hndl&
   '           stat&  0=OK
   
   ' Create, configure, and open an Encoder channel.
   ' stat& = 0 if the program exits successfully
   
   On Error Resume Next
   stat& = 0
   
   If Dev(indx&).hndl <> 0 Then Exit Sub  ' Skip if already opened
      
   ' ===================================
   ' Create an instance of a Phidget channel.  Once created the
   ' channel may need to be configured to match the physical
   ' device and channel you want to attach to, and then be
   ' opened by calling Phidget_open().  The channel handle must
   ' be released with PhidgetEncoder_delete().
   '
   ' stat& (Error Code) will always be 0 if no Errors
   '
   stat& = PhTempCreate&(Dev(indx&).hndl) ' Create new handle
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET SERIAL NUMBER
   stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
   If stat& <> 0 Then GoTo HandleErr

   ' SET ISHUB (0 if hub port is connected to a Phidget module)
   stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
   If stat& <> 0 Then GoTo HandleErr

   ' SET HUB PORT
   stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
   If stat& <> 0 Then GoTo HandleErr

   ' SET CHANNEL
   stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
   If stat& <> 0 Then GoTo HandleErr
   
   ' Setup the handlers to be run when the device is plugged in,
   ' opened from software, unplugged, closed from software, or
   ' generates an error.
'   ' SETUP ATTACH EVENT
'   stat& = PhSetOnAttach&(Dev(indx&).hndl, _
'               AddressOf PhCommon22.PhOnAttach, _
'               indx&) ' returned with event for this handle
'   If stat& <> 0 Then GoTo HandleErr
'
'   ' SETUP DETACH EVENT
'   stat& = PhSetOnDetach&(Dev(indx&).hndl, _
'               AddressOf PhCommon22.PhOnDetach, _
'               indx&) ' returned with event for this handle
'   If stat& <> 0 Then GoTo HandleErr
'
'   ' SETUP ERROR EVENT
'   ' Using common Error Handler for all phidget modules.
'   stat& = PhSetOnError&(Dev(indx&).hndl, _
'               AddressOf PhCommon22.PhOnError, _
'               indx&) ' returned with event for this handle
'   If stat& <> 0 Then GoTo HandleErr
'
'   ' SETUP ON TEMPERATURE CHANGE EVENT
'   stat& = PhTempSetOnTemperatureChangeHandler( _
'               Dev(indx&).hndl, _
'               AddressOf PhVintTempOnTemperatureChange&, _
'               indx&) ' returned with event for this handle
'   If stat& <> 0 Then GoTo HandleErr

   ' OPEN CHANNEL
   'stat& = PhOpen(Dev(indx&).hndl)
   'If stat& <> 0 Then GoTo HandleErr
      
   ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
   stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, VintTimeOutMs&)
   If stat& <> 0 Then GoTo HandleErr
      
   Exit Sub

HandleErr:
   Display.AlertLbl = "A029  InitPhVintTemp" & vbCrLf _
                    & PhGetErrorDescription$(stat&)

End Sub
   
'===========================================================================
Public Static Sub SetupPhVintTemp(indx&, stat&)

   Dim attached&
      
   ' Setup initial pramaters (Must be attached first)
   PhGetAttached& Dev(indx&).hndl, attached&
   If attached& = 0 Then stat& = &H34: GoTo HandleErr ' Device not Attached
      
   ' SET TEMPERATURE CHANGE TRIGGER ' Default: 0.0 C
   ' The channel will not issue a TemperatureChange event until the
   ' temperature value has changed by the amount specified by the
   ' TemperatureChangeTrigger.
   ' Setting the TemperatureChangeTrigger to 0 will result in
   ' the channel firing events every DataInterval.  This is useful
   ' for applications that implement their own data filtering
   stat& = PhTempSetTempChangeTrigger(Dev(indx&).hndl, 0&)
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET TEMPERATURE DATA INTERVAL (Default 500ms)
   ' The dataInterval is the time that must elapse before the channel will
   ' fire another TemperatureChange event.
   ' The data interval is bounded by MinDataInterval and MaxDataInterval.
   ' The timing between TemperatureChange events can also affected by
   ' the TemperatureChangeTrigger.
   stat& = PhTempSetDataInterval(Dev(indx&).hndl, 500&)
   If stat& <> 0 Then GoTo HandleErr
   
   Exit Sub

HandleErr:
   Display.AlertLbl = "A030  SetupPhVintTemp" & vbCrLf _
                    & PhGetErrorDescription$(stat&)

End Sub


